@@ -1,5 +1,4 @@
 from __future__ import annotations
-
 import inspect
 import logging
 import types
@@ -14,16 +13,16 @@ from dataclasses import (
     make_dataclass,
 )
 from typing import Any, Optional, Union
-
+from functools import partial
+from enum import EnumMeta
 from orjson import OPT_INDENT_2
-
 from datamodel.converters import parse_type
 from datamodel.fields import Field
 from datamodel.types import JSON_TYPES
 from datamodel.validation import validator
 
 from .exceptions import ValidationError
-from .parsers.encoders import DefaultEncoder
+from .parsers.encoders import DefaultEncoder, json_encoder
 
 
 class Meta:
@@ -31,6 +30,7 @@ class Meta:
     Metadata information about Model.
     """
     name: str = ""
+    description: str = ""
     schema: str = ""
     app_label: str = ""
     frozen: bool = False
@@ -233,11 +233,12 @@ class BaseModel(metaclass=ModelMeta):
             raise TypeError(
                 f'Cannot create a new field {name} on a Strict Model.'
             )
-        f = Field(required=False, default=value)
-        f.name = name
-        f.type = type(value)
-        self.__columns__[name] = f
-        setattr(self, name, value)
+        if name != '__errors__':
+            f = Field(required=False, default=value)
+            f.name = name
+            f.type = type(value)
+            self.__columns__[name] = f
+            setattr(self, name, value)
 
     def set(self, name: str, value: Any) -> None:
         """set.
@@ -247,7 +248,7 @@ class BaseModel(metaclass=ModelMeta):
             value (Any): value to be assigned.
         """
         if name not in self.__columns__:
-            if self.Meta.strict is False: # can be created new Fields
+            if name != '__errors__' and self.Meta.strict is False: # can be created new Fields
                 self.create_field(name, value)
         else:
             setattr(self, name, value)
@@ -265,8 +266,10 @@ class BaseModel(metaclass=ModelMeta):
                 encoder = f.metadata['encoder']
             else:
                 encoder = None
+            if hasattr(f, 'default') and self.is_callable(value):
+                continue
             ### Factory Value:
-            if isinstance(f.type, types.MethodType):
+            elif isinstance(f.type, types.MethodType):
                 raise TypeError(
                     f"DataModel: Wrong type for Column {key}: {f.type}"
                 )
@@ -312,10 +315,13 @@ class BaseModel(metaclass=ModelMeta):
                 elif self.is_empty(value):
                     is_missing = isinstance(f.default, _MISSING_TYPE)
                     setattr(self, key, f.default_factory if is_missing else f.default)
-                elif new_val:= parse_type(f.type, value, encoder):
-                    # be processed by _parse_type
-                    setattr(self, key, new_val)
                 else:
+                    try:
+                        # be processed by _parse_type
+                        new_val= parse_type(f.type, value, encoder)
+                        setattr(self, key, new_val)
+                    except (TypeError, ValueError):
+                        pass
                     continue
         try:
             self._validation()
@@ -324,7 +330,8 @@ class BaseModel(metaclass=ModelMeta):
 
     def is_callable(self, value) -> bool:
         is_missing = (value == _MISSING_TYPE)
-        return callable(value) if not is_missing else False
+        is_function = isinstance(value, (types.BuiltinFunctionType, types.FunctionType, partial))
+        return callable(value) if not is_missing and is_function else False
 
     def is_empty(self, value) -> bool:
         if isinstance(value, _MISSING_TYPE):
@@ -381,6 +388,9 @@ class BaseModel(metaclass=ModelMeta):
                     pass
                 try:
                     if f.metadata["required"] is True and self.Meta.strict is True:
+                        # if this has db_default:
+                        if 'db_default' in f.metadata:
+                            continue
                         raise ValueError(
                             f"::{self.modelName}:: Missing Required Field *{name}*"
                         )
@@ -463,9 +473,9 @@ class BaseModel(metaclass=ModelMeta):
                 if _type.__module__ == 'typing':
                     # TODO: discover real value of typing
                     if _type._name == 'List':
-                        t = 'list'
+                        t = 'array'
                     elif _type._name == 'Dict':
-                        t = 'dict'
+                        t = 'object'
                     else:
                         try:
                             t = _type.__args__[0]
@@ -479,11 +489,139 @@ class BaseModel(metaclass=ModelMeta):
                         t = 'object'
                 cols[key] = {"name": key, "type": t}
             doc = {
+                "$schema": "https://json-schema.org/draft/2020-12/schema",
+                "$id": f"/schemas/{table}",
                 "name": clsname,
                 "description": cls.__doc__.strip("\n").strip(),
+                "additionalProperties": False,
                 "table": table,
                 "schema": schema,
-                "fields": cols,
+                "type": "object",
+                "properties": cols,
             }
             result = cls.__encoder__.dumps(doc, option=OPT_INDENT_2)
         return result
+
+    @classmethod
+    def schema(cls, as_dict: bool = False) -> Any:
+        """schema.
+
+        Get JSON Schema of Current Model.
+        Returns: str: string (json) version of model.
+
+        TODO: using Nested Models to create the $ref/schemas/{name}
+        * Using $defs to define sub-schemas based on custom types.
+        * using "definitions" to create "enum" of enum fields.
+        """
+        title = cls.__name__
+        schema = cls.Meta.schema
+        table = cls.Meta.name if cls.Meta.name else title.lower()
+        columns = cls.columns(cls).items()
+        description = cls.__doc__.strip("\n").strip()
+        if not description:
+            description = cls.Meta.description
+        fields = {}
+        required = []
+        defs = {}
+        for name, field in columns:
+            _type = field.type
+            if _type.__module__ == 'typing':
+                # TODO: discover real value of typing
+                if _type._name == 'List':
+                    t = 'array'
+                elif _type._name == 'Dict':
+                    t = 'object'
+                else:
+                    try:
+                        t = _type.__args__[0]
+                        t = t.__name__
+                    except (AttributeError, ValueError):
+                        t = 'string'
+            else:
+                if isinstance(_type, EnumMeta):
+                    t = 'array'
+                    enum_type = {
+                        "type": "string",
+                        "enum": list(map(lambda c: c.value, _type))
+                    }
+                elif isinstance(_type, ModelMeta):
+                    t = 'object'
+                    ref = f"/schemas/{_type.__name__}"
+                    sch = _type.schema(as_dict = True)
+                    defs[name] = sch
+                else:
+                    ref = None
+                    enum_type = None
+                    try:
+                        t = JSON_TYPES[_type]
+                    except KeyError:
+                        t = 'string'
+            ## check of min and max:
+            minimum = field.metadata.get('min', None)
+            maximum = field.metadata.get('max', None)
+            # secret:
+            secret = field.metadata.get('secret', None)
+            label = field.metadata.get('label', None)
+            try:
+                if field.metadata["required"] is True or field.metadata['primary'] is True:
+                    required.append(name)
+            except KeyError:
+                pass
+            fields[name] = {
+                "type": t,
+                "nullable": field.metadata.get('nullable', False),
+                "label": label,
+                "attrs": {
+                    "placeholder": field.metadata.get('description', None),
+                    "format": field.metadata.get('format', None),
+                },
+                "readOnly": field.metadata.get('readonly', False),
+                "writeOnly": False
+            }
+            if 'pattern' in field.metadata:
+                fields[name]["attrs"]["pattern"] = field.metadata['pattern']
+            if enum_type:
+                fields[name]["items"] = enum_type
+            if ref:
+                fields[name]["$ref"] = ref
+            # check if field need to be represented:
+            if field.repr is False:
+                fields[name]["attrs"]["visible"] = False
+            # if 'default' in field.metadata:
+            fields[name]['default'] = field.default
+            if secret is not None:
+                fields[name]['secret'] = secret
+            if t == 'string':
+                if minimum:
+                    fields[name]['minLength'] = minimum
+                if maximum:
+                    fields[name]['maxLength'] = maximum
+            else:
+                if minimum:
+                    fields[name]['minimum'] = minimum
+                if maximum:
+                    fields[name]['maximum'] = maximum
+            if field.metadata['widget']:
+                fields[name]['widget'] = field.metadata['widget']
+        if cls.Meta.strict is True:
+            adp = True
+        else:
+            adp = False
+        base_schema = {
+            "$schema": "https://json-schema.org/draft/2020-12/schema",
+            "$id": f"/schemas/{table}",
+            "additionalProperties": adp,
+            "title": title,
+            "description": description,
+            "type": "object",
+            "table": table,
+            "schema": schema,
+            "properties": fields,
+            "required": required
+        }
+        if defs:
+            base_schema["$defs"] = defs
+        if as_dict is True:
+            return base_schema
+        else:
+            return json_encoder(base_schema)
